import { E2EBenutzerHelper } from 'apps/admin-e2e/src/helper/benutzer/benutzer.helper';
import { E2EBenutzerVerifier } from 'apps/admin-e2e/src/helper/benutzer/benutzer.verifier';
import { OrganisationsEinheitE2E } from 'apps/admin-e2e/src/model/organisations-einheit';
import { AdminUserE2E } from 'apps/admin-e2e/src/model/util';
import { loginAsAriane } from '../../../support/user-util';

describe('Organisationseinheit zu Benutzer hinzufügen', () => {
  const benutzerVerifier: E2EBenutzerVerifier = new E2EBenutzerVerifier();
  const benutzerHelper: E2EBenutzerHelper = new E2EBenutzerHelper();

  const vorname: string = 'Theo';
  const nachname: string = 'Testuser';

  const now1 = new Date();
  const userName1: string = 'testtheo' + now1.getSeconds().toString() + now1.getMilliseconds().toString();
  const emailAddress1: string = 'theo' + now1.getSeconds().toString() + now1.getMilliseconds().toString() + '@ozg-sh.de';

  const now2 = new Date(now1.getTime() + 1000);
  const userName2: string = 'testtheo' + now2.getSeconds().toString() + now2.getMilliseconds().toString();
  const emailAddress2: string = 'theo' + now2.getSeconds().toString() + now2.getMilliseconds().toString() + '@ozg-sh.de';

  const userWithoutOrganisationsEinheiten: AdminUserE2E = {
    vorname: vorname,
    nachname: nachname,
    username: userName1,
    email: emailAddress1,
    isUser: true,
    organisationseinheiten: [],
  };

  const userWithOrganisationsEinheiten: AdminUserE2E = {
    vorname: vorname,
    nachname: nachname,
    username: userName2,
    email: emailAddress2,
    isUser: true,
    organisationseinheiten: [OrganisationsEinheitE2E.DENKMALPFLEGE],
  };

  before(() => {
    loginAsAriane();
  });

  it('should show organisationseinheiten in formular', () => {
    benutzerHelper.openNewBenutzerPage();

    benutzerVerifier.verifyOrganisationsEinheitenInFormular([
      OrganisationsEinheitE2E.DENKMALPFLEGE,
      OrganisationsEinheitE2E.ORDNUNGSAMT,
    ]);
  });

  it('should add new user without organisationseinheiten', () => {
    benutzerHelper.openNewBenutzerPage();

    benutzerHelper.addBenutzerAndSave(userWithoutOrganisationsEinheiten);

    benutzerVerifier.verifyNoOrganisationsEinheitInBenutzerList(userName1);
  });

  it('should add new user with organisationseinheiten', () => {
    benutzerHelper.openNewBenutzerPage();

    benutzerHelper.addBenutzerAndSave(userWithOrganisationsEinheiten);

    benutzerVerifier.verfiyOrganisationsEinheitInBenutzerList(OrganisationsEinheitE2E.DENKMALPFLEGE, userName2);
  });

  it('should remove organisationseinheit from existing user', () => {
    benutzerHelper.openBenutzerPage(userName2);

    benutzerHelper.editOrganisationsEinheitenAndSave([OrganisationsEinheitE2E.DENKMALPFLEGE]);

    benutzerVerifier.verifyNoOrganisationsEinheitInBenutzerList(userName2);
  });

  it('should add organisationseinheit to existing user', () => {
    benutzerHelper.openBenutzerPage(userName2);

    benutzerHelper.editOrganisationsEinheitenAndSave([OrganisationsEinheitE2E.ORDNUNGSAMT]);

    benutzerVerifier.verfiyOrganisationsEinheitInBenutzerList(OrganisationsEinheitE2E.ORDNUNGSAMT, userName2);
  });
});
