/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { convertToDataTestId } from '../../support/tech.util';
import { AttachmentContainerE2EComponent } from '../attachment/attachment.e2e.component';
import { UserProfileE2EComponent } from '../user-profile/user-profile.component.e2e';

export class KommentarListInVorgangE2EComponent {
  readonly hinzufuegenButton: string = 'create-kommentar';

  readonly root: string = 'kommentar-container-in-vorgang';

  readonly kommentarList: string = 'kommentar-list-in-vorgang';

  readonly formular: string = 'kommentar-formular';
  readonly formularItem: string = 'kommentar-list-item-form';
  readonly formularSpeichernButton: string = 'add-button';
  readonly formularAbbrechenButton: string = 'cancel-button';
  readonly textInput: string = 'Kommentar-textarea-input';
  readonly textError: string = 'Kommentar-textarea-error';

  private readonly attachmentContainer: AttachmentContainerE2EComponent = new AttachmentContainerE2EComponent();

  public getRoot(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.root);
  }

  public getFormular(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.formular);
  }

  public getFormulare(): Cypress.Chainable<Element[]> {
    return cy.getTestElement(this.formularItem);
  }

  public getHinzufuegenButton(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.hinzufuegenButton);
  }

  public getFormularSpeichernButton(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.formularSpeichernButton);
  }

  public getFormularAbbrechenButton(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.formularAbbrechenButton);
  }

  public getKommentarList(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.kommentarList);
  }

  public getTextInput(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.textInput);
  }

  public getTextError(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.textError);
  }

  public getKommentar(text: string): KommentarInVorgangE2EComponent {
    return new KommentarInVorgangE2EComponent(text);
  }

  public getAttachmentContainer(): AttachmentContainerE2EComponent {
    return this.attachmentContainer;
  }
}

export class KommentarInVorgangE2EComponent {
  private readonly root: string;

  private readonly createdAtInput: string = 'kommentar-created-at';

  constructor(private text: string) {
    this.root = 'kommentar-item-' + convertToDataTestId(this.text);
  }

  public getRoot(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.root);
  }

  public getUserProfile(): UserProfileE2EComponent {
    return new UserProfileE2EComponent(this.root);
  }

  public getCreatedAt(): Cypress.Chainable<Element> {
    return this.getRoot().findTestElementWithClass(this.createdAtInput);
  }
}
