/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { APP_HELPER } from 'apps/alfa-e2e/src/helper/app.const';
import { VORGANG_EXECUTOR, VORGANG_NAVIGATOR, VORGANG_VERIFIER } from 'apps/alfa-e2e/src/helper/vorgang/vorgang.const';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E } from '../../../model/vorgang';
import { dropCollections } from '../../../support/cypress-helper';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

describe('Vorgang wiedereroeffnen', () => {
  const vorgangBeschiedenWiedereroeffnen: VorgangE2E = {
    ...createVorgang(),
    name: 'DoWiedereroeffnenOnBeschieden',
    status: VorgangStatusE2E.BESCHIEDEN,
  };
  const vorgangBeschiedenWiedereroeffnenRevoke: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'DoRevokeWiedereroeffnenOnBeschieden'),
    status: VorgangStatusE2E.BESCHIEDEN,
  };
  const vorgangAbgeschlossenWiedereroeffnen: VorgangE2E = {
    ...buildVorgang(objectIds[1], 'DoWiedereroeffnenOnAbgeschlossen'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
  };
  const vorgangAbgeschlossenWiedereroeffnenRevoke: VorgangE2E = {
    ...buildVorgang(objectIds[2], 'DoRevokeWiedereroeffnenOnAbgeschlossen'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
  };

  before(() => {
    initVorgaenge([
      vorgangBeschiedenWiedereroeffnen,
      vorgangBeschiedenWiedereroeffnenRevoke,
      vorgangAbgeschlossenWiedereroeffnen,
      vorgangAbgeschlossenWiedereroeffnenRevoke,
    ]);

    APP_HELPER.loginAsSabine();
  });

  after(() => {
    dropCollections();
  });

  describe('vorgang in status beschieden', () => {
    it('should show vorgang status', () => {
      VORGANG_NAVIGATOR.openVorgang(vorgangBeschiedenWiedereroeffnen.name);

      VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.BESCHIEDEN);
    });

    describe('wiedereroeffnen', () => {
      describe('by button', () => {
        it('should change status to wiedereroeffnet', () => {
          VORGANG_EXECUTOR.wiedereroeffnenByButton();

          VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.IN_BEARBEITUNG);
        });

        it('should have new status in list after navigate back', () => {
          VORGANG_NAVIGATOR.navigateBack();

          VORGANG_VERIFIER.verifyStatusInList(vorgangBeschiedenWiedereroeffnen.name, VorgangStatusE2E.IN_BEARBEITUNG);
        });
      });
    });

    describe('wiedereroeffnen und rückgängig machen', () => {
      it('should show vorgang status', () => {
        VORGANG_NAVIGATOR.openVorgang(vorgangBeschiedenWiedereroeffnenRevoke.name);

        VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.BESCHIEDEN);
      });

      describe('by icon-button', () => {
        it('should have status Beschieden', () => {
          VORGANG_EXECUTOR.wiedereroeffnenByIconButton();

          APP_HELPER.revokeAndCloseSnackbar(VorgangMessagesE2E.WIEDEREROEFFNET);

          VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.BESCHIEDEN);
        });

        it('should have reverted status in list after navigate back', () => {
          VORGANG_NAVIGATOR.navigateBack();

          VORGANG_VERIFIER.verifyStatusInList(vorgangBeschiedenWiedereroeffnenRevoke.name, VorgangStatusE2E.BESCHIEDEN);
        });
      });
    });
  });

  describe('vorgang in status abgeschlossen', () => {
    describe('wiedereroeffnen', () => {
      it('should show vorgang status', () => {
        VORGANG_NAVIGATOR.openVorgang(vorgangAbgeschlossenWiedereroeffnen.name);

        VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.ABGESCHLOSSEN);
      });

      describe('wiedereroeffnen', () => {
        describe('by button', () => {
          it('should change status to wiedereroeffnet', () => {
            VORGANG_EXECUTOR.wiedereroeffnenByButton();

            VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.IN_BEARBEITUNG);
          });

          it('should have new status in list after navigate back', () => {
            VORGANG_NAVIGATOR.navigateBack();

            VORGANG_VERIFIER.verifyStatusInList(vorgangAbgeschlossenWiedereroeffnen.name, VorgangStatusE2E.IN_BEARBEITUNG);
          });
        });
      });

      describe('wiedereroeffnen und rückgängig machen', () => {
        it('should show vorgang status', () => {
          VORGANG_NAVIGATOR.openVorgang(vorgangAbgeschlossenWiedereroeffnenRevoke.name);

          VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.ABGESCHLOSSEN);
        });

        describe('by icon-button', () => {
          it('should have status Beschieden', () => {
            VORGANG_EXECUTOR.wiedereroeffnenByIconButton();

            APP_HELPER.revokeAndCloseSnackbar(VorgangMessagesE2E.WIEDEREROEFFNET);

            VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.ABGESCHLOSSEN);
          });

          it('should have reverted status in list after navigate back', () => {
            VORGANG_NAVIGATOR.navigateBack();

            VORGANG_VERIFIER.verifyStatusInList(vorgangAbgeschlossenWiedereroeffnenRevoke.name, VorgangStatusE2E.ABGESCHLOSSEN);
          });
        });
      });
    });
  });
});
