/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { ApiError, createEmptyStateResource } from '@alfa-client/tech-shared';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Action } from '@ngrx/store';
import { createCommandResource, createSuccessfullyDoneCommandStateResource } from 'libs/command-shared/test/command';
import { createApiError } from 'libs/tech-shared/test/error';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { BescheidState, initialState, reducer } from './bescheid.reducer';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';

describe('Bescheid Reducer', () => {
  describe('unknown action', () => {
    it('should return current state', () => {
      const action: Action = {} as Action;

      const result = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('bescheidCommand', () => {
    describe('on "createCommand" action', () => {
      it('should set resource loading', () => {
        const resource: VorgangWithEingangResource = createVorgangWithEingangResource();
        const action: Action = CommandActions.createCommand({
          resource,
          linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
          command: { ...createCommandResource(), order: CommandOrder.CREATE_BESCHEID },
        });

        const state: BescheidState = reducer(initialState, action);

        expect(state.bescheidCommand).toEqual(createEmptyStateResource(true));
      });

      it('should replace state resource with loading', () => {
        const initialState: BescheidState = { bescheidCommand: createSuccessfullyDoneCommandStateResource() };
        const resource: VorgangWithEingangResource = createVorgangWithEingangResource();
        const action: Action = CommandActions.createCommand({
          resource,
          linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
          command: { ...createCommandResource(), order: CommandOrder.CREATE_BESCHEID },
        });

        const state: BescheidState = reducer(initialState, action);

        expect(state.bescheidCommand).toEqual(createEmptyStateResource(true));
      });
    });

    describe('on "createCommandSuccess" action', () => {
      it('should set resource', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.CREATE_BESCHEID,
        };
        const action: Action = CommandActions.createCommandSuccess({ command });

        const state: BescheidState = reducer(initialState, action);

        expect(state.bescheidCommand.resource).toBe(command);
      });
    });

    describe('on "createCommandFailure" action', () => {
      const command: CommandResource = {
        ...createCommandResource(),
        order: CommandOrder.CREATE_BESCHEID,
      };

      it('should set error', () => {
        const apiError: ApiError = createApiError();
        const action: Action = CommandActions.createCommandFailure({
          command,
          error: { error: apiError },
        });

        const state: BescheidState = reducer(initialState, action);

        expect(state.bescheidCommand.error).toBe(apiError);
      });
    });
  });
});
