/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandProps, CommandResource, CreateCommandFailureProps, CreateCommandProps } from '@alfa-client/command-shared';
import { ApiError, createEmptyStateResource, createErrorStateResource, createStateResource, StateResource, } from '@alfa-client/tech-shared';
import { HttpErrorResponse } from '@angular/common/http';
import { Action, ActionReducer, createReducer, on } from '@ngrx/store';
import { isCreateBescheidCommand } from '../bescheid.util';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';

export const BESCHEID_FEATURE_KEY = 'BescheidState';

export interface BescheidPartialState {
  readonly [BESCHEID_FEATURE_KEY]: BescheidState;
}

export interface BescheidState {
  bescheidCommand: StateResource<CommandResource>;
}

export const initialState: BescheidState = {
  bescheidCommand: createEmptyStateResource(),
};

const bescheidReducer: ActionReducer<BescheidState, Action> = createReducer(
  initialState,
  on(CommandActions.createCommand, (state: BescheidState, props: CreateCommandProps): BescheidState => {
    return isCreateBescheidCommand(props.command.order) ? { ...state, bescheidCommand: createEmptyStateResource(true) } : state;
  }),
  on(CommandActions.createCommandSuccess, (state: BescheidState, props: CommandProps): BescheidState => {
    return isCreateBescheidCommand(props.command.order) ?
        { ...state, bescheidCommand: createStateResource(props.command) }
      : state;
  }),
  on(CommandActions.createCommandFailure, (state: BescheidState, props: CreateCommandFailureProps): BescheidState => {
    return isCreateBescheidCommand(props.command.order) ?
        {
          ...state,
          bescheidCommand: createErrorStateResource(<ApiError>(<HttpErrorResponse>props.error).error),
        }
      : state;
  }),
);

export function reducer(state: BescheidState, action: Action): BescheidState {
  return bescheidReducer(state, action);
}
