/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import { CommandOrder, CreateCommand, CreateCommandProps } from '@alfa-client/command-shared';
import { EMPTY_STRING, StateResource, getEmbeddedResources } from '@alfa-client/tech-shared';
import { getUrl } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { BescheidLinkRel, BescheidListLinkRel } from './bescheid.linkrel';
import {
  Bescheid,
  BescheidListResource,
  BescheidResource,
  UploadFileInProgress,
} from './bescheid.model';

export function buildCreateBescheidCommand(bescheid: Bescheid = null): CreateCommand {
  return { order: CommandOrder.CREATE_BESCHEID, body: bescheid };
}

export function isCreateBescheidCommand(order: CommandOrder): boolean {
  return order == CommandOrder.CREATE_BESCHEID;
}

function buildCreateBescheidDocumentFromFileCommand(binaryFile: BinaryFileResource): CreateCommand {
  return {
    order: CommandOrder.CREATE_BESCHEID_DOCUMENT_FROM_FILE,
    body: {
      documentFile: getUrl(binaryFile),
    },
  };
}

export function buildCreateBescheidDocumentFromFileProps(
  bescheid: BescheidResource,
  binaryFile: BinaryFileResource,
): CreateCommandProps {
  return {
    resource: bescheid,
    linkRel: BescheidLinkRel.CREATE_DOCUMENT_FROM_FILE,
    command: buildCreateBescheidDocumentFromFileCommand(binaryFile),
    snackBarMessage: EMPTY_STRING,
  };
}

export function buildUpdateBescheidCommandProps(
  resource: BescheidResource,
  bescheid: Bescheid,
): CreateCommandProps {
  return {
    resource,
    linkRel: BescheidLinkRel.UPDATE,
    command: {
      order: CommandOrder.UPDATE_BESCHEID,
      body: bescheid,
    },
    snackBarMessage: EMPTY_STRING,
  };
}

function buildDeleteBescheidCommand(): CreateCommand {
  return { order: CommandOrder.DELETE_BESCHEID, body: null };
}

export function buildDeleteBescheidCommandProps(resource: BescheidResource): CreateCommandProps {
  return <any>{
    resource,
    linkRel: BescheidLinkRel.DELETE,
    command: buildDeleteBescheidCommand(),
    snackBarMessage: EMPTY_STRING,
  };
}

export function buildCreateBescheidDocumentCommandProps(
  resource: BescheidResource,
): CreateCommandProps {
  return {
    resource,
    linkRel: BescheidLinkRel.CREATE_DOCUMENT,
    command: {
      order: CommandOrder.CREATE_BESCHEID_DOCUMENT,
      body: null,
    },
    snackBarMessage: EMPTY_STRING,
  };
}

export function buildSendBescheidCommandProps(
  resource: BescheidResource,
  linkRel: string,
): CreateCommandProps {
  return {
    resource,
    linkRel,
    command: {
      order: CommandOrder.SEND_BESCHEID,
      body: null,
    },
    snackBarMessage: EMPTY_STRING,
  };
}

export function getEmbeddedBescheidResources(
  bescheidListStateResource: StateResource<BescheidListResource>,
) {
  return getEmbeddedResources<BescheidResource>(
    bescheidListStateResource,
    BescheidListLinkRel.BESCHEID_LIST,
  );
}

export function isUploadFinished(uploadInProgress: UploadFileInProgress): boolean {
  return !uploadInProgress.loading;
}

export function hasUploadNoError(uploadInProgress: UploadFileInProgress): boolean {
  return isNil(uploadInProgress.error);
}
