/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { faker } from '@faker-js/faker';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';
import { createBinaryFileResource } from '../../../binary-file-shared/test/binary-file';
import { createSuccessfullyDoneCommandStateResource } from '../../../command-shared/test/command';
import { createProblemDetail } from '../../../tech-shared/test/error';
import { BescheidListLinkRel } from '../lib/bescheid.linkrel';
import { Bescheid, BescheidAttachments, BescheidDocument, BescheidListResource, BescheidResource, BescheidSendBy, BescheidStatus, BescheidWizardStep, UploadFileInProgress, Wizard, } from '../lib/bescheid.model';

export function createBescheid(): Bescheid {
  return {
    status: BescheidStatus.DRAFT,
    beschiedenAm: '2024-01-01',
    bewilligt: true,
    bescheidDocument: faker.internet.url(),
    attachments: [faker.internet.url(), faker.internet.url()],
    sendBy: BescheidSendBy.NACHRICHT,
    nachrichtSubject: faker.lorem.text(),
    nachrichtText: faker.lorem.text(),
  };
}

export function createBescheidResource(linkRel: string[] = []): BescheidResource {
  return toResource(createBescheid(), linkRel);
}

export function createBescheidStateResource(linkRel: string[] = []): StateResource<BescheidResource> {
  return createStateResource(createBescheidResource(linkRel));
}

export function createUploadFileInProgress(): UploadFileInProgress {
  return { loading: true, fileName: faker.person.firstName(), error: createProblemDetail() };
}

export function createBescheidResources(linkRelations: string[] = []): BescheidResource[] {
  return times(10, () => toResource(createBescheidResource(), [...linkRelations]));
}

export function createBescheidListResource(
  bescheide: BescheidResource[] = createBescheidResources(),
  linkRelations: string[] = [],
): BescheidListResource {
  return toResource({}, [...linkRelations], {
    [BescheidListLinkRel.BESCHEID_LIST]: bescheide,
  });
}

export function createBescheidAttachments(): BescheidAttachments {
  return {
    upload: createUploadFileInProgress(),
    items: [createBinaryFileResource()],
    uploadStateResource: createStateResource(createBinaryFileResource()),
  };
}

export function createBescheidDocument(): BescheidDocument {
  return {
    upload: createUploadFileInProgress(),
    create: createSuccessfullyDoneCommandStateResource(),
    documentUri: faker.internet.url(),
    resource: createBinaryFileResource(),
  };
}

export function createSuccessfulUpload(): UploadFileInProgress {
  return {
    loading: false,
    error: null,
    fileName: faker.system.fileName(),
  };
}

export function createPendingUpload(): UploadFileInProgress {
  return {
    loading: true,
    fileName: faker.system.fileName(),
    error: null,
  };
}

export function createErrorUpload(): UploadFileInProgress {
  return {
    loading: false,
    error: createProblemDetail(),
    fileName: faker.system.fileName(),
  };
}

export function createWizard(): Wizard {
  return {
    empfaenger: faker.person.fullName(),
    sendBy: BescheidSendBy.MANUAL,
    bescheidCreated: false,
    canBeSend: true,
    activeStep: BescheidWizardStep.BescheidVersenden,
  };
}
