import { BescheidResource, BescheidSendBy } from '@alfa-client/bescheid-shared';
import { existsAsHtmlElement, getElementComponentFromFixtureByCss, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';
import { createBescheidResource, createWizard } from '../../../../../../../bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { BescheidWizardBescheidVersendenSummaryComponent } from './bescheid-wizard-bescheid-versenden-summary.component';
import { BescheidWizardBescheidVersendenSendenComponent } from './senden/bescheid-wizard-bescheid-versenden-senden.component';
import { BescheidWizardBescheidVersendenSpeichernComponent } from './speichern/bescheid-wizard-bescheid-versenden-speichern.component';

describe('BescheidWizardBescheidVersendenSummaryComponent', () => {
  let component: BescheidWizardBescheidVersendenSummaryComponent;
  let fixture: ComponentFixture<BescheidWizardBescheidVersendenSummaryComponent>;

  const bescheidVersendenSpeichernTestId: string = getDataTestIdOf('bescheid-versenden-speichern');
  const bescheidVersendenSendenTestId: string = getDataTestIdOf('bescheid-versenden-senden');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardBescheidVersendenSummaryComponent,
        MockComponent(BescheidWizardBescheidVersendenSpeichernComponent),
        MockComponent(BescheidWizardBescheidVersendenSendenComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardBescheidVersendenSummaryComponent);
    component = fixture.componentInstance;
    component.wizard = createWizard();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    const bescheidResource: BescheidResource = createBescheidResource();

    describe('bescheid versenden speichern', () => {
      it('should exists', () => {
        component.wizard = { ...createWizard(), sendBy: BescheidSendBy.MANUAL };

        fixture.detectChanges();

        existsAsHtmlElement(fixture, bescheidVersendenSpeichernTestId);
      });

      it('should NOT exists', () => {
        component.wizard = { ...createWizard(), sendBy: BescheidSendBy.NACHRICHT };

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidVersendenSpeichernTestId);
      });

      it('should have inputs', () => {
        component.wizard = { ...createWizard(), sendBy: BescheidSendBy.MANUAL };
        component.bescheidResource = bescheidResource;

        fixture.detectChanges();
        const speichernComponent: BescheidWizardBescheidVersendenSpeichernComponent =
          getElementComponentFromFixtureByCss<BescheidWizardBescheidVersendenSpeichernComponent>(
            fixture,
            bescheidVersendenSpeichernTestId,
          );

        expect(speichernComponent.bescheidResource).toEqual(bescheidResource);
      });
    });

    describe('bescheid versenden senden', () => {
      it('should exists', () => {
        component.wizard = { ...createWizard(), sendBy: BescheidSendBy.NACHRICHT };

        fixture.detectChanges();

        existsAsHtmlElement(fixture, bescheidVersendenSendenTestId);
      });

      it('should NOT exists', () => {
        component.wizard = { ...createWizard(), sendBy: BescheidSendBy.MANUAL };

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidVersendenSendenTestId);
      });

      it('should have inputs', () => {
        component.wizard = { ...createWizard(), sendBy: BescheidSendBy.NACHRICHT };
        component.bescheidResource = bescheidResource;

        fixture.detectChanges();
        const sendenComponent: BescheidWizardBescheidVersendenSendenComponent =
          getElementComponentFromFixtureByCss<BescheidWizardBescheidVersendenSendenComponent>(
            fixture,
            bescheidVersendenSendenTestId,
          );

        expect(sendenComponent.bescheidResource).toEqual(bescheidResource);
        expect(sendenComponent.wizard).toEqual(component.wizard);
      });
    });
  });
});
