import {
  BinaryFileListLinkRel,
  BinaryFileListResource,
  BinaryFileResource,
  BinaryFileService,
  UploadFileByIdentifier,
} from '@alfa-client/binary-file-shared';
import { getEmbeddedResources, isLoaded, StateResource } from '@alfa-client/tech-shared';
import { AsyncPipe } from '@angular/common';
import { Component, inject, Input } from '@angular/core';
import { hasLink, Resource } from '@ngxp/rest';
import { filter, map, Observable, startWith, switchMap } from 'rxjs';
import { tap } from 'rxjs/operators';
import { BinaryFileListOrientation } from '../directive/binary-file-list-orientation/binary-file-list-orientation.directive';
import { FileUploadListComponent } from './file-upload-list/file-upload-list.component';

@Component({
  selector: 'ods-file-upload-list-container',
  standalone: true,
  templateUrl: './file-upload-list-container.component.html',
  imports: [FileUploadListComponent, AsyncPipe],
})
export class FileUploadListContainerComponent {
  @Input() fileUploadType: string;
  @Input() parentFormArrayName: string;
  @Input() listOrientation: BinaryFileListOrientation = BinaryFileListOrientation.HORIZONTAL;

  @Input() set filesResource(value: Resource) {
    this._filesResource = value;
    this._initFileList();
  }

  @Input() set filesLinkRel(value: string) {
    this._filesLinkRel = value;
    this._initFileList();
  }

  private readonly binaryFileService: BinaryFileService = inject(BinaryFileService);

  public uploadedFiles$: Observable<UploadFileByIdentifier>;
  _filesResource: Resource;
  _filesLinkRel: string;

  _initFileList(): void {
    if (hasLink(this._filesResource, this._filesLinkRel)) {
      this.uploadedFiles$ = this.binaryFileService.getFiles(this._filesResource, this._filesLinkRel).pipe(
        filter(isLoaded),
        map((files: StateResource<BinaryFileListResource>) => getEmbeddedResources(files, BinaryFileListLinkRel.FILE_LIST)),
        tap((binaryFileResources: BinaryFileResource[]) =>
          this.binaryFileService.addFiles(this.fileUploadType, binaryFileResources),
        ),
        switchMap(() => this.binaryFileService.getUploadedFiles(this.fileUploadType)),
        startWith({}),
      );
    } else {
      this.uploadedFiles$ = this.binaryFileService.getUploadedFiles(this.fileUploadType);
    }
  }

  public onDelete(key: string): void {
    this.binaryFileService.deleteUploadedFile(this.fileUploadType, key);
  }
}
