/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CollaborationListResource } from '@alfa-client/collaboration-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, getMockComponent, mock } from '@alfa-client/test-utils';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CollaborationService } from 'libs/collaboration-shared/src/lib/collaboration.service';
import { createCollaborationListResource } from 'libs/collaboration-shared/test/collaboration';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { CollaborationInVorgangContainerComponent } from './collaboration-in-vorgang-container.component';
import { CollaborationInVorgangComponent } from './collaboration-in-vorgang/collaboration-in-vorgang.component';
import { CollaborationListContainerComponent } from './collaboration-list-container/collaboration-list-container.component';

describe('CollaborationInVorgangContainerComponent', () => {
  let component: CollaborationInVorgangContainerComponent;
  let fixture: ComponentFixture<CollaborationInVorgangContainerComponent>;

  let service: Mock<CollaborationService>;

  const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(async () => {
    service = mock(CollaborationService);
    await TestBed.configureTestingModule({
      declarations: [
        CollaborationInVorgangContainerComponent,
        MockComponent(CollaborationListContainerComponent),
        MockComponent(CollaborationInVorgangComponent),
      ],
      providers: [
        {
          provide: CollaborationService,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(CollaborationInVorgangContainerComponent);
    component = fixture.componentInstance;
    component.vorgang = vorgang;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service to get collaboration state list resource', () => {
      component.ngOnInit();

      expect(service.getList).toHaveBeenCalled();
    });

    it('should call service to get organisationsEinheit anfrage button visbility', () => {
      component.ngOnInit();

      expect(service.showOrganisationsEinheitAnfrageButton).toHaveBeenCalled();
    });

    it('should call service to get externeFachstelle anfrage button visbility', () => {
      component.ngOnInit();

      expect(service.showExterneFachstelleAnfrageButton).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    describe('collaboration in vorgang component', () => {
      const collaborationListResource: CollaborationListResource = createCollaborationListResource();
      const collaborationStateListResource: StateResource<CollaborationListResource> =
        createStateResource(collaborationListResource);

      describe('should be called with', () => {
        it('vorgang', () => {
          component.vorgang = vorgang;

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().vorgang).toBe(vorgang);
        });

        it('collaboration state list resource', () => {
          component.collaborationStateListResource$ = of(collaborationStateListResource);

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().collaborationStateListResource).toBe(collaborationStateListResource);
        });

        it('showOrganisationsEinheitAnfrageButton', () => {
          component.showOrganisationsEinheitAnfrageButton$ = of(true);

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().showOrganisationsEinheitAnfrageButton).toBeTruthy();
        });

        it('showExterneFachstelleAnfrageButton', () => {
          component.showExterneFachstelleAnfrageButton$ = of(true);

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().showExterneFachstelleAnfrageButton).toBeTruthy();
        });

        function getCollaborationInVorgangComponent(): CollaborationInVorgangComponent {
          return getMockComponent<CollaborationInVorgangComponent>(fixture, CollaborationInVorgangComponent);
        }
      });
    });
  });
});
