/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { AttachmentWrapperComponent } from '../attachment-wrapper/attachment-wrapper.component';
import { AttachmentComponent } from '../attachment/attachment.component';
import { BescheidStatusTextComponent } from '../bescheid-status-text/bescheid-status-text.component';
import { BescheidWrapperComponent } from './bescheid-wrapper.component';

const meta: Meta<BescheidWrapperComponent> = {
  title: 'Wrappers/Bescheid wrapper',
  component: BescheidWrapperComponent,
  parameters: {
    docs: {
      description: {
        component: 'Wrapper for bescheid information',
      },
    },
  },
  decorators: [
    moduleMetadata({
      imports: [
        AttachmentComponent,
        AttachmentWrapperComponent,
        BescheidWrapperComponent,
        BescheidStatusTextComponent,
      ],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<BescheidWrapperComponent>;

export const Default: Story = {
  render: () => ({
    template: `<ods-bescheid-wrapper>
        <ods-bescheid-status-text
            bewilligt="true"
            dateText="13.11.2024"
            [hasBescheidDraft]="false"
        ></ods-bescheid-status-text>
        <ods-attachment-wrapper>
            <ods-attachment caption="Attachment" description="200 kB" fileType="pdf"></ods-attachment>
            <ods-attachment caption="Second attachment" description="432 kB" fileType="doc"></ods-attachment>
        </ods-attachment-wrapper>
    </ods-bescheid-wrapper>`,
  }),
};
