/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { argsToTemplate, componentWrapperDecorator, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';
import { TooltipDirective } from './tooltip.directive';

const meta: Meta = {
  title: 'Tooltip',
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
  decorators: [
    componentWrapperDecorator((story) => `<div class="flex justify-between mt-16">${story}</div>`),
    moduleMetadata({
      imports: [TooltipDirective],
    }),
  ],
  parameters: {
    docs: {
      description: {
        component: 'Tooltip directive that can be used with every element (check out default story to see tooltip working).',
      },
    },
  },
};

export default meta;
type Story = StoryObj;

export const Default: Story = {
  args: { tooltip: 'I stay in the center', tooltipPosition: 'below', tooltipAriaType: 'aria-describedby' },
  argTypes: {
    tooltip: { description: 'Tooltip text' },
    tooltipPosition: {
      description: 'Tooltip position dependent on parent element',
      control: 'select',
      options: ['above', 'below'],
    },
    tooltipAriaType: {
      description: 'If true sets aria-describedby attribute to parent element, otherwise aria-labeledby',
      control: 'select',
      options: ['aria-describedby', 'aria-labeledby'],
      table: { defaultValue: { summary: 'aria-describedby' } },
    },
  },
  render: (args) => ({
    props: args,
    template: `
      <button tooltip="I am displayed to the right">→</button>
      <button ${argsToTemplate(args)}>Hover me</button>
      <button tooltip="I am displayed to the left">←</button>
    `,
  }),
};

export const Above: Story = {
  render: () => ({
    template: `
      <button tooltip="I am displayed to the right" tooltipPosition="above">→</button>
      <button tooltip="I stay in the center" tooltipPosition="above">Hover me</button>
      <button tooltip="I am displayed to the left" tooltipPosition="above">←</button>
    `,
  }),
};
