/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiError, ApiErrorAction, createStateResource } from '@alfa-client/tech-shared';
import { Action } from '@ngrx/store';
import { createApiError } from '../../../../tech-shared/test/error';
import { createHintListResource } from '../../../test/hint';
import { HintListResource } from '../hint.model';
import { HintListProps } from './hint.actions';
import { HintState, initialState, reducer } from './hint.reducer';

import * as HintActions from './hint.actions';

describe('HintReducer', () => {
  describe('unknown action', () => {
    it('should return current state', () => {
      const action = {} as Action;

      const result = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('on loadHintListSuccess action', () => {
    it('should set loaded resource into state', () => {
      const hintList: HintListResource = createHintListResource();
      const action = HintActions.loadHintListSuccess(<HintListProps>{ hintList });

      const result: HintState = reducer(initialState, action);

      expect(result.hintList).toEqual(createStateResource(hintList));
    });
  });

  describe('on loadHintListFailed action', () => {
    it('should set error into state', () => {
      const apiError: ApiError = createApiError();
      const action = HintActions.loadHintListFailed(<ApiErrorAction>{ apiError });

      const result: HintState = reducer(initialState, action);

      expect(result.hintList.error).toBe(apiError);
    });
  });
});
