/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { mock, useFromMock } from '@alfa-client/test-utils';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createHintListResource } from 'libs/hint-shared/test/hint';
import { createApiRootResource } from '../../../../../libs/api-root-shared/test/api-root';
import { HintListResource } from '../hint.model';
import { HintRepository } from './hint.repository';

describe('HintRepository', () => {
  let service: HintRepository;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { get: jest.fn() };

  beforeEach(() => {
    service = new HintRepository(useFromMock(resourceFactory));

    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('loadAll', () => {
    const hintListResource: HintListResource = createHintListResource();
    const apiRoot: ApiRootResource = createApiRootResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: hintListResource }));
    });

    it('should call resourceFactory with given resource', () => {
      service.loadAll(apiRoot);

      expect(resourceFactory.from).toHaveBeenCalledWith(apiRoot);
    });

    it('should call resourceWrapper with link', () => {
      service.loadAll(apiRoot);

      expect(resourceWrapper.get).toHaveBeenCalledWith(ApiRootLinkRel.HINTS);
    });

    it('should return result', () => {
      const result = service.loadAll(apiRoot);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: hintListResource }));
    });
  });
});
