/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createLoeschAnforderungResource } from 'libs/loesch-anforderung-shared/test/loesch-anforderung';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import { initialState, LoeschAnforderungPartialState } from './loesch-anforderung.reducer';

import * as LoeschAnforderungSelectors from './loesch-anforderung.selectors';

describe('LoeschAnforderung Selectors', () => {
  let state: LoeschAnforderungPartialState;

  const loeschenAnfordernCommand: StateResource<CommandResource> =
    createStateResource(createCommandResource());
  const loeschAnforderung: StateResource<LoeschAnforderungResource> = createStateResource(
    createLoeschAnforderungResource(),
  );
  const vorgangLoeschenCommand: StateResource<CommandResource> =
    createStateResource(createCommandResource());
  const loeschAnforderungZuruecknehmenCommand: StateResource<CommandResource> =
    createStateResource(createCommandResource());

  beforeEach(() => {
    state = {
      LoeschAnforderungState: {
        ...initialState,

        loeschenAnfordernCommand,
        loeschAnforderung,
        vorgangLoeschenCommand,
        loeschAnforderungZuruecknehmenCommand,
      },
    };
  });

  it('should return loeschenAnfordernCommand', () => {
    const result: StateResource<CommandResource> =
      LoeschAnforderungSelectors.loeschenAnfordernCommand.projector(state.LoeschAnforderungState);

    expect(result).toBe(loeschenAnfordernCommand);
  });

  it('should return loeschAnforderung from state', () => {
    const result: StateResource<LoeschAnforderungResource> =
      LoeschAnforderungSelectors.loeschAnforderung.projector(state.LoeschAnforderungState);

    expect(result).toBe(loeschAnforderung);
  });

  it('should return vorgangLoeschenCommand from state', () => {
    const result: StateResource<CommandResource> =
      LoeschAnforderungSelectors.vorgangLoeschenCommand.projector(state.LoeschAnforderungState);

    expect(result).toBe(vorgangLoeschenCommand);
  });

  it('should return loeschAnforderungZuruecknehmenCommand', () => {
    const result: StateResource<CommandResource> =
      LoeschAnforderungSelectors.loeschAnforderungZuruecknehmenCommand.projector(
        state.LoeschAnforderungState,
      );

    expect(result).toBe(loeschAnforderungZuruecknehmenCommand);
  });
});
