/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { provideMockActions } from '@ngrx/effects/testing';
import { routerNavigatedAction } from '@ngrx/router-store';
import { Action } from '@ngrx/store';
import { provideMockStore } from '@ngrx/store/testing';
import { hot } from 'jest-marbles';
import { Observable } from 'rxjs';
import { createRouteData } from './../../../test/navigation-test-factory';
import { NavigationEffects } from './navigation.effects';

import * as NavigationUtil from '../navigation.util';
import * as NavigationActions from './navigation.actions';

describe('NavigationEffects', () => {
  let actions: Observable<Action>;
  let effects: NavigationEffects;

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [NavigationEffects, provideMockActions(() => actions), provideMockStore()],
    });

    effects = TestBed.inject(NavigationEffects);
  });

  describe('navigate$', () => {
    const action = routerNavigatedAction;

    it('should dispatch updateCurrentRouteData action with data', () => {
      jest.spyOn(NavigationUtil, 'buildRouteData').mockReturnValue(createRouteData());
      actions = hot('-a-|', { a: action });

      effects.navigateEnd$.subscribe();

      const expected = hot('-a-|', {
        a: NavigationActions.updateCurrentRouteData({ routeData: createRouteData() }),
      });

      expect(effects.navigateEnd$).toBeObservable(expected);
    });
  });
});
