/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { PostfachMailFormDialogData, PostfachMailResource } from '@alfa-client/postfach-shared';
import { createStateResource } from '@alfa-client/tech-shared';
import { Mock, dispatchEventFromFixture, getMockComponent, mock } from '@alfa-client/test-utils';
import { DialogService, OzgcloudIconComponent } from '@alfa-client/ui';
import { getEmpfaenger } from '@alfa-client/vorgang-shared-ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { PostfachMailFormComponent } from 'libs/postfach/src/lib/postfach-mail-form/postfach-mail-form.component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { PostfachNachrichtEditButtonContainerComponent } from './postfach-nachricht-edit-button-container.component';

jest.mock('@alfa-client/vorgang-shared-ui');
const getEmpfaengerMock = getEmpfaenger as jest.Mock;

describe('PostfachNachrichtEditButtonContainerComponent', () => {
  let component: PostfachNachrichtEditButtonContainerComponent;
  let fixture: ComponentFixture<PostfachNachrichtEditButtonContainerComponent>;

  const postfachNachrichtEditButton: string = getDataTestIdOf('postfach-nachricht-edit-button');

  const dialogService: Mock<DialogService> = mock(DialogService);

  const postfachNachricht: PostfachMailResource = createPostfachMailResource();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        PostfachNachrichtEditButtonContainerComponent,
        MockComponent(OzgcloudIconComponent),
      ],
      providers: [
        {
          provide: DialogService,
          useValue: dialogService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(PostfachNachrichtEditButtonContainerComponent);
    component = fixture.componentInstance;
    component.postfachNachricht = postfachNachricht;
    component.vorgangStateResource = createStateResource(createVorgangWithEingangResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ozgcloud icon component', () => {
    it('should be called with icon', () => {
      const ozgcloudIconComponent: OzgcloudIconComponent = getMockComponent(
        fixture,
        OzgcloudIconComponent,
      );

      expect(ozgcloudIconComponent.icon).toBe('edit');
    });
  });

  describe('on click on button', () => {
    it('should call dialogService', () => {
      dispatchEventFromFixture(fixture, postfachNachrichtEditButton, 'click');

      expect(dialogService.openFixed).toHaveBeenCalled();
    });

    it('should build dialogData', () => {
      component.buildDialogData = jest.fn();

      dispatchEventFromFixture(fixture, postfachNachrichtEditButton, 'click');

      expect(component.buildDialogData).toHaveBeenCalled();
    });
  });

  describe('buildDialogData', () => {
    it('should have title', () => {
      const dialogData: PostfachMailFormDialogData = component.buildDialogData();

      expect(dialogData.title).toBe('Nachricht bearbeiten');
    });

    it('should have component', () => {
      const dialogData: PostfachMailFormDialogData = component.buildDialogData();

      expect(dialogData.component).toBe(PostfachMailFormComponent);
    });

    it('should have postfachNachricht', () => {
      const dialogData: PostfachMailFormDialogData = component.buildDialogData();

      expect(dialogData.postfachNachricht).toBe(postfachNachricht);
    });

    it('should get empfänger', () => {
      component.buildDialogData();

      expect(getEmpfaengerMock).toHaveBeenCalled();
    });

    it('should set empfänger', () => {
      const empfanger: string = faker.person.firstName();
      getEmpfaengerMock.mockReturnValue(empfanger);

      const dialogData: PostfachMailFormDialogData = component.buildDialogData();

      expect(dialogData.empfaenger).toEqual(empfanger);
    });
  });
});
