/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidWizardContainerComponent, BescheidWizardDialogData } from '@alfa-client/bescheid';
import { BescheidWizardDialogResult } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { createEmptyStateResource, isNotNil, StateResource } from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangCommandService, VorgangService, VorgangWithEingangLinkRel, VorgangWithEingangResource, } from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { Component, inject, Input, OnInit, ViewContainerRef } from '@angular/core';
import { hasLink } from '@ngxp/rest';
import { Observable, of } from 'rxjs';

@Component({
  selector: 'alfa-bescheiden-button',
  templateUrl: './bescheiden-button.component.html',
  styles: [':host {@apply empty:hidden}'],
})
export class BescheidenButtonComponent implements OnInit {
  @Input() vorgang: VorgangWithEingangResource;
  @Input() showAsIconButton: boolean = false;

  viewContainerRef = inject(ViewContainerRef);
  private readonly vorgangCommandService = inject(VorgangCommandService);
  private readonly vorgangService = inject(VorgangService);
  private readonly ozgcloudDialogService = inject(OzgcloudDialogService);

  commandStateResource$: Observable<StateResource<CommandResource>> = of(createEmptyStateResource<CommandResource>());

  readonly linkRel = VorgangWithEingangLinkRel;

  get buttonText(): string {
    return hasLink(this.vorgang, VorgangWithEingangLinkRel.BESCHEID_DRAFT) ? 'Bescheiden fortsetzen' : 'Bescheiden';
  }

  get toolTipText(): string {
    return hasLink(this.vorgang, VorgangWithEingangLinkRel.BESCHEID_DRAFT) ? 'Vorgang bescheiden fortsetzen' : (
        'Vorgang bescheiden'
      );
  }

  ngOnInit(): void {
    this.commandStateResource$ = this.vorgangCommandService.getBeschiedenCommand();
  }

  public bescheiden(): void {
    this.openBescheidenWizard();
  }

  public openBescheidenWizard(): void {
    const dialogData: BescheidWizardDialogData = { vorgangWithEingangResource: this.vorgang };
    const dialogRef: DialogRef<BescheidWizardDialogResult> = this.ozgcloudDialogService.openWizard<
      BescheidWizardContainerComponent,
      BescheidWizardDialogData,
      BescheidWizardDialogResult
    >(BescheidWizardContainerComponent, this.viewContainerRef, dialogData);
    dialogRef.closed.subscribe((dialogResult: BescheidWizardDialogResult) => this.handleBescheidWizardClosed(dialogResult));
  }

  handleBescheidWizardClosed(dialogResult: BescheidWizardDialogResult): void {
    if (isNotNil(dialogResult) && dialogResult.reloadVorgang) {
      this.vorgangService.reloadCurrentVorgang();
    }
  }
}
