import { ComponentFixture, TestBed } from '@angular/core/testing';

import { CommandResource, isSuccessfulDone } from '@alfa-client/command-shared';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { VorgangArchiveConfirmationDialogContainerComponent } from './vorgang-archive-confirmation-dialog-container.component';
import { VorgangArchiveConfirmationDialogComponent } from './vorgang-archive-confirmation-dialog/vorgang-archive-confirmation-dialog.component';

jest.mock('@alfa-client/command-shared');

describe('VorgangArchiveConfirmationDialogContainerComponent', () => {
  let component: VorgangArchiveConfirmationDialogContainerComponent;
  let fixture: ComponentFixture<VorgangArchiveConfirmationDialogContainerComponent>;

  const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
  const dialogRef = { ...mock(DialogRef), config: { data: { vorgangWithEingang } } };

  const vorgangArchiveState: StateResource<boolean> = createStateResource(true);
  const vorgangService = {
    ...mock(VorgangService),
    getVorgangArchive: jest.fn().mockReturnValue(of(createStateResource(vorgangArchiveState))),
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangArchiveConfirmationDialogContainerComponent,
        MockComponent(VorgangArchiveConfirmationDialogComponent),
      ],
      providers: [
        { provide: DialogRef, useValue: dialogRef },
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangArchiveConfirmationDialogContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('archiveDoneHandler', () => {
      it('should close dialog', () => {
        component.archiveDoneHandler();

        expect(dialogRef.close).toHaveBeenCalled();
      });
    });

    describe('onCancelButtonClicked', () => {
      it('should close dialog', () => {
        component.onCancelButtonClicked();

        expect(dialogRef.close).toHaveBeenCalled();
      });
    });

    describe('onConfirmationButtonClicked', () => {
      const commandState: StateResource<CommandResource> = createStateResource(createCommandResource());

      beforeEach(() => {
        component.archiveDoneHandler = jest.fn();
        vorgangService.archive.mockReturnValue(of(commandState));
      });

      it('should call vorgangService archive', () => {
        component.onConfirmationButtonClicked();

        expect(vorgangService.archive).toHaveBeenCalledWith(vorgangWithEingang);
      });

      it('should call archiveDoneHandler if command was successfull', () => {
        (isSuccessfulDone as jest.Mock).mockReturnValue(true);

        component.onConfirmationButtonClicked();
        component.archiveStateResource$.subscribe();

        expect(component.archiveDoneHandler).toHaveBeenCalled();
      });

      it('should not call archiveDoneHandler if command was not successfull', () => {
        (isSuccessfulDone as jest.Mock).mockReturnValue(false);

        component.onConfirmationButtonClicked();
        component.archiveStateResource$.subscribe();

        expect(component.archiveDoneHandler).not.toHaveBeenCalled();
      });
    });
  });
});
